/*
 * ATtiny85 based firmware ring modulator with sound to light
 *
 * Created by Mike Harper, October 2015
 *
 * The ATtiny85 should be configured to run off the internal PLL clock at 16MHz.
 *
 * Microphone input is aquired through physical pin 7 (ADC1) with Vcc (5V) as the reference.
 *
 * Modulation frequency is varied by a potentiometer connected to physical pin 3 (ADC2).
 * The potentiometer should be configured with a fixed resistor divider so the input
 * voltage varies from 0->2.5V to maintain compatibility with the Digispark and Trinket.
 *
 * Voice-to-light is controlled by physical pin 6 (PB1).  Brightness is PWM at ~65kHz,
 * and the pin is intended to drive an n-channel MOSFET through a ~1kOhm resistor.
 * 
 * Modulated voice output is on physical pin 5 (PB0).  Signal is PWM at ~65kHz, and is
 * intended to be filtered before amplification.
 *
 * This code was written simply for the challenge of making a minimalistic voice
 * modulator and is made public with the thought someone might find it useful.  No
 * warranty or offer of support is implied.
 *
 *
 * software gun code added by Chris Adams 2016
 */
 
int val;           // Signal amplitude
int val2;          // Dummy variable
int lights;        // Light brightness
int count=0;       // Loop counter controls the modulation frequency
int count_max=30;  // Initialized to a max of 30 for mid range modulation
int sine_i=0;      // sine wave index counter
int sine_i_max=27; // the sine wave is broken into 27 values
int  TCCR1set;
int  TCCR0Aset;
int  TCCR0Bset;
int  ADCSRAset;
int  ADCSRBset;

int sine[]={0, 7, 13, 19, 24, 28, 30, 31, 30, 28, 24, 19, 13, 7, // The sine wave
            0,-7,-13,-19,-24,-28,-30,-31,-30,-28,-24,-19,-13,-7};

void setup() {
  noInterrupts();
  ADCSRAset=ADCSRA;
  ADCSRBset=ADCSRB;
  TCCR1set=TCCR1;
  TCCR0Aset=TCCR0A;
  TCCR0Bset=TCCR0B;
  setupADCforVoice(); // Sets up the ADC to collect the voice
  
  // Setup PWM Light DAC on PB1
  DDRB  |= (1<<DDB1);                // Set PB1 as output (Physical pin 6)
  TCCR1 = (1<<CS10);                 // Clear Timer 1 settings and set Pre-scale = 1
  TCCR1 |= (1<<PWM1A) | (1<<COM1A1); // Enable PWM on PB1 w/ OC1A clear on compare match
  OCR1A = 0;                         // Lights off
  
  // Setup PWM Audio DAC on PB0
  DDRB   |= (1<<DDB0);               // Set PB0 as output (Physical pin 5)
  TCCR0A = TCCR0B = 0;               // Clear Timer0 settings
  TCCR0A |= (1<<WGM00) | (1<<WGM01); // Fast PWM Mode
  TCCR0A |= (1<<COM0A1);             // Clear on compare match
  TCCR0B |= (1<<CS00);               // Pre-scale = 1
  OCR0A = 127;                       // Set to "zero"

  pinMode(3, INPUT);  /* gun trigger */ 
  
  interrupts();
}

// reset everything after gun is fired
void setup2(){
  noInterrupts();
  setupADCforVoice(); // Sets up the ADC to collect the voice
  
  // Setup PWM Light DAC on PB1
  DDRB  |= (1<<DDB1);                // Set PB1 as output (Physical pin 6)
  TCCR1 = (1<<CS10);                 // Clear Timer 1 settings and set Pre-scale = 1
  TCCR1 |= (1<<PWM1A) | (1<<COM1A1); // Enable PWM on PB1 w/ OC1A clear on compare match
  OCR1A = 0;                         // Lights off
  
  // Setup PWM Audio DAC on PB0
  DDRB   |= (1<<DDB0);               // Set PB0 as output (Physical pin 5)
  TCCR0A = TCCR0B = 0;               // Clear Timer0 settings
  TCCR0A |= (1<<WGM00) | (1<<WGM01); // Fast PWM Mode
  TCCR0A |= (1<<COM0A1);             // Clear on compare match
  TCCR0B |= (1<<CS00);               // Pre-scale = 1
  OCR0A = 127;                       // Set to "zero"

  pinMode(3, INPUT);  /* gun trigger */ 
  
  interrupts();
}

void loop() {
  lights = constrain(2*abs(val-128),0,255);
  if (lights > 85) OCR1A = lights;   // Threshold to prevent the lights from
  else OCR1A = 0;                    // flickering when the voice is silent

    // gun tone sequence triggered by pushbutton switch between pin 5 and +5V
//  pinMode(3, INPUT);  /* gun trigger */ 

    // gun trigger pressed
    if (digitalRead(3) == HIGH) {
      delay(50);    // debounce switch
      while (digitalRead(3) == HIGH) {
        while (digitalRead(3) == HIGH) {  // wait for trigger button to be released before firing
        }
        newBlast();
        digitalWrite(3, 255);   // turn on gun light
        delay(8);        
        digitalWrite(3, 0);  // turn off gun light
        delay(12);
      } // end -button debounced
      setup2(); 
    } // end -trigger pressed

}

void newBlast(){
int qq = 5;
int blst = 250;
int bstep = 18;
int dly = 4;
int jj;
int blast;
int freq;
    pinMode(0, OUTPUT);         // play tone for qq 
    noInterrupts();
    TCCR0A = TCCR0Aset;         // Reset to get tones working
    TCCR0B = TCCR0Bset;        
    TCCR1 = TCCR1set;        
    ADCSRA = ADCSRAset;
    ADCSRB = ADCSRBset;
    interrupts();
        while(qq--){
          digitalWrite(3, 255);   // turn on gun light
          blast=0;
          while(blast++<=bstep){   // 1 cycle = 6280 microseconds
            freq=(blst*2)+(blast*25);
            tone(0,freq);
            delay(dly);
            noTone(0);
            delay(8);
            if (blast > 8)
               digitalWrite(3, 0);  // turn off gun light
          }    // end of 1 cycle
       }  // end of qq cycles
       // one last blast - slower
          digitalWrite(3, 255);   // turn on gun light
          blast=0;
          bstep=8;
          qq=0;
          while(blast++<=bstep){   // 1 cycle = 6280 microseconds
            freq=(blst*2)+(blast*25);
            jj=0;
            while (++jj < 2){
              tone(0,freq);
              delay(dly);
              noTone(0);
              delay(qq+8);
              if (blast >8)
                 digitalWrite(3, 0);  // turn off gun light
            }
            //++qq;
          }    // end of 1 cycle

        noTone(0);
}

ISR(ADC_vect) {
  if (sine_i == 0) {                        // When the modulation amplitude is zero, update the
    if (count == 0) {                       // modulation frequency.
      setupADCforFrequency();               // Switch the ADC channel to read physical pin 3 (ADC2)
      val2 = ADCH;                          // Read ADCH to trigger the next conversion
      OCR0A = 127;                          // "zero" because sine[0] = 0
    } else if (count == 3) {                // skip the first few conversions after the switch
      val2 = ADCH;                          // Read ADCH to trigger the next conversion
      count_max = constrain(map(ADCH,0,127,15,75),15,75); // read the modulation frequency select on physical pin 3
      OCR0A = 127;                          // "zero" because sine[0] = 0
    } else if (count == 4) {                
      setupADCforVoice();                   // sets the ADC back to collect voice
      val2 = ADCH;                          // Read ADCH to trigger the next conversion
      OCR0A = 127;                          // "zero" because sine[0] = 0
    } else {
      val2 = ADCH;                          // Read ADCH to trigger the next conversion
      OCR0A = 127;                          // "zero" because sine[0] = 0
    }
  } else {
      val = ADCH;                                // Read the highest 8 bits from the ADC
      if ((val > 112) && (val < 142)) val = 127; // Noise gate
      OCR0A = ((val-127)*sine[sine_i])/32+127;   // A sine amplitude over 32 would have caused an int overflow
  }
  count++;                                  // increment the loop counter
  if (count >= count_max) {                 
    count = 0;                              // if the counter expires reset the counter
    sine_i++;                               // and increment the sine wave index
  }
  if (sine_i == sine_i_max) sine_i = 0;     // Loop the sine wave index when it reaches the end
}

void setupADCforFrequency() {        // Sets the ADC registers to measure the
  ADMUX  = B00100010;                // voltage on physical pin 3 (ADC2) which is used
  ADCSRA = B11101110;                // to select the modulation frequency.
  ADCSRB = B00000000;
}

void setupADCforVoice() {            // Sets the ADC registers to measure the
  ADMUX  = B00100001;                // microphone signal on physical pin 7 (ADC1)
  ADCSRA = B11101110;                // with Vcc as the reference voltage.
  ADCSRB = B00000000;
}

